package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.entity.TBSopInfo;
import com.qmth.sop.business.mapper.SopScheduleMapper;
import com.qmth.sop.business.service.SopScheduleService;
import com.qmth.sop.common.enums.SopAnalyseGroupEnum;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class SopScheduleServiceImpl extends ServiceImpl<SopScheduleMapper, TBSopInfo> implements SopScheduleService {
    /**
     * * ①准备：扫描前的准备阶段；
     * * ②扫描：扫描仪配置产出物提交，到扫描收尾产出物提交；
     * * ③评卷：阅卷参数提交、评卷收尾；
     * * ④收尾：项目总结提交前；
     * * ⑤已完结：SOP完结。
     */
    @Override
    public Map<String, Map<String, Object>> progress(Long serviceId, SopAnalyseGroupEnum group) {
        List<Map<String, Object>> list = this.baseMapper.progress(serviceId);
        Map<String, Map<String, Object>> result = new HashMap<>();
        if (group.equals(SopAnalyseGroupEnum.POPULATION)) {
            result.put("POPULATION", processData(list));
        } else if (group.equals(SopAnalyseGroupEnum.REGION)) {
            list.stream().collect(Collectors.groupingBy(map -> map.get("region_name"))).forEach((k, v) -> result.put(k.toString(), processData(v)));
        } else if (group.equals(SopAnalyseGroupEnum.SUPPLIER)) {
            list.stream().collect(Collectors.groupingBy(map -> map.get("name"))).forEach((k, v) -> result.put(k.toString(), processData(v)));
        }
        return result;
    }

    private Map<String, Object> processData(List<Map<String, Object>> list) {
        Map<String, Object> map = new HashMap<>();
        map.put("prepare", list.stream().filter(map1 -> map1.get("setup") == null || (Integer.parseInt(map1.get("setup").toString()) > 0 && Integer.parseInt(map1.get("setup").toString()) < 5)).count());
        map.put("scan", list.stream().filter(map1 -> map1.get("setup") != null && "5,6,7".indexOf(map1.get("setup").toString()) > 0).count());
        map.put("evaluation", list.stream().filter(map1 -> map1.get("setup") != null && "8,9".indexOf(map1.get("setup").toString()) > 0).count());
        map.put("summary", list.stream().filter(map1 -> map1.get("setup") != null && (Integer.parseInt(map1.get("setup").toString()) == 11 || Integer.parseInt(map1.get("setup").toString()) == 10)).count());
        map.put("finish", list.stream().filter(map1 -> map1.get("setup") != null && Integer.parseInt(map1.get("setup").toString()) == 0).count());
        return map;
    }

    @Override
    public <T> IPage<Map<String, Object>> detail(Page<T> tPage, Long serviceId, Long regionId, Long supplierId) {
        return this.baseMapper.detail(tPage, serviceId, regionId, supplierId);
    }

}
