package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.dto.DataPermissionDto;
import com.qmth.sop.business.bean.dto.DateFormDto;
import com.qmth.sop.business.bean.dto.SopCrmInfo;
import com.qmth.sop.business.bean.params.DingSaveParam;
import com.qmth.sop.business.bean.result.*;
import com.qmth.sop.business.entity.*;
import com.qmth.sop.business.mapper.TBDingMapper;
import com.qmth.sop.business.service.*;
import com.qmth.sop.business.sync.FaceApiUtils;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.*;
import com.qmth.sop.common.util.DateDisposeUtils;
import com.qmth.sop.common.util.FileStoreUtil;
import com.qmth.sop.common.util.FileUtil;
import com.qmth.sop.common.util.ServletUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 考勤打卡表 服务实现.
 *
 * @author: shudonghui
 * @date: 2023-08-10 14:23:08
 * @version: 1.0
 * @email: shudonghui@qmth.com.cn
 * @Company: www.qmth.com.cn
 */
@Service
public class TBDingServiceImpl extends ServiceImpl<TBDingMapper, TBDing> implements TBDingService {
    @Resource
    private SysRoleService sysRoleService;
    @Resource
    private TBCrmService tbCrmService;
    @Resource
    private TBUserArchivesService tbUserArchivesService;
    @Resource
    private SysDingDateService sysDingDateService;
    @Resource
    private TBSopInfoService tbSopInfoService;
    @Resource
    private FileStoreUtil fileStoreUtil;
    @Resource
    private FaceApiUtils faceApiUtils;
    @Resource
    private SysConfigService sysConfigService;
    @Resource
    private TBDingHistoryService tbDingHistoryService;
    @Resource
    private SysUserService sysUserService;

    @Override
    public DingElementResult findDingElements(String sopNo, Long userId) {

        SopCrmInfo sopCrmInfo = tbCrmService.findSopCrmInfoBySop(sopNo);
        Long serviceUnitId = sopCrmInfo.getServiceUnitId();

        DingElementResult dingElementResult = this.baseMapper.findDingResultByUserId(userId, serviceUnitId);
        if (Objects.nonNull(dingElementResult)) {
            Long userArchivesId = dingElementResult.getUserArchivesId();
            String currentDayStr = DateFormatUtils.format(System.currentTimeMillis(), SystemConstant.DEFAULT_DATE_DAY_PATTERN);
            List<TBDing> tbDingList = this.list(new QueryWrapper<TBDing>()
                    .lambda()
                    .eq(TBDing::getSopNo, sopNo)
                    .eq(TBDing::getUserArchivesId, userArchivesId)
                    .eq(TBDing::getSignDate, currentDayStr));
            if (CollectionUtils.isNotEmpty(tbDingList)) {
                // 有考勤记录
                if (tbDingList.size() > 1) {
                    throw ExceptionResultEnum.ERROR.exception("考勤记录异常");
                }
                TBDing tbDing = tbDingList.get(0);
                Long signInTime = tbDing.getSignInTime();
                String singInAddress = tbDing.getSignInAddress();
                Long signOutTime = tbDing.getSignOutTime();
                String signOutAddress = tbDing.getSignOutAddress();

                dingElementResult.setSignInTime(signInTime);
                dingElementResult.setSignInAddress(singInAddress);
                dingElementResult.setSignOutAddress(signOutAddress);
                dingElementResult.setSignOutTime(signOutTime);
                dingElementResult.setDingSopNo(tbDing.getDingSopNo());
            }
        }
        return dingElementResult;
    }

    @Override
    public DingStatisticResult findDingStatistic(String sopNo, Long userId) {
        // 根据sopNo查询日期表单
        DateFormDto dateFormDto = this.buildSignDateFormBySop(sopNo);
        List<String> dateFormList = dateFormDto.getDateFormList();


        int workCount = 0;
        int exceptionCount = 0;
        List<DingFormResult> formList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(dateFormList)) {

            List<TBDing> tbDingList = this.list(new QueryWrapper<TBDing>().lambda()
                    .eq(TBDing::getSopNo, sopNo)
                    .eq(TBDing::getUserId, userId)
                    .orderByAsc(TBDing::getId));

            // 签到表
            List<DingFormResult> formDingList = new ArrayList<>();
            for (TBDing tbDing : tbDingList) {
                String signDate = tbDing.getSignDate();
                Long signInTime = tbDing.getSignInTime();
                String signInAddress = tbDing.getSignInAddress();
                String axisInX = tbDing.getAxisInX();
                String axisInY = tbDing.getAxisInY();
                String dingSopNo = tbDing.getDingSopNo();

                DingInfoResult signInInfo = new DingInfoResult();
                if (signInTime == null || signInTime == 0) {
                    signInInfo.setStatus(DingStatusEnum.NO_SIGN);
                } else {
                    if (sopNo.equals(dingSopNo)) {
                        signInInfo.setStatus(DingStatusEnum.SIGN);
                    } else {
                        signInInfo.setStatus(DingStatusEnum.OTHER);
                    }
                    signInInfo.setInOutType(InOutTypeEnum.IN);
                    signInInfo.setSignTime(signInTime);
                    signInInfo.setSignAddress(signInAddress);
                    signInInfo.setAxisX(axisInX);
                    signInInfo.setAxisY(axisInY);
                }

                Long signOutTime = tbDing.getSignOutTime();
                String signOutAddress = tbDing.getSignOutAddress();
                String axisOutX = tbDing.getAxisOutX();
                String axisOutY = tbDing.getAxisOutY();

                DingInfoResult signOutInfo = new DingInfoResult();
                if (signOutTime == null || signOutTime == 0) {
                    signOutInfo.setStatus(DingStatusEnum.NO_SIGN);
                } else {
                    if (sopNo.equals(dingSopNo)) {
                        signOutInfo.setStatus(DingStatusEnum.SIGN);
                    } else {
                        signOutInfo.setStatus(DingStatusEnum.OTHER);
                    }
                    signOutInfo.setInOutType(InOutTypeEnum.OUT);
                    signOutInfo.setSignTime(signOutTime);
                    signOutInfo.setSignAddress(signOutAddress);
                    signOutInfo.setAxisX(axisOutX);
                    signOutInfo.setAxisY(axisOutY);
                }
                DingFormResult formResult = new DingFormResult();
                formResult.setDingId(tbDing.getId());
                formResult.setSignDate(signDate);
                formResult.setSignInInfo(signInInfo);
                formResult.setSignOutInfo(signOutInfo);
                formDingList.add(formResult);
            }

            String endDate = dateFormDto.getEndDate();

            for (String date : dateFormList) {
                String year = date.substring(0, 4);
                String signDate = date.substring(5);

                List<DingFormResult> dingFormResultList = formDingList.stream()
                        .filter(e -> signDate.equals(e.getSignDate()))
                        .collect(Collectors.toList());

                DingFormResult dingFormResult = new DingFormResult();
                if (CollectionUtils.isNotEmpty(dingFormResultList)) {
                    // 有这一天的打卡记录
                    dingFormResult = dingFormResultList.get(0);
                    // 考勤异常天数增加
                    if (dingFormResult.getSignInInfo().getStatus().equals(DingStatusEnum.NO_SIGN)) {
                        if (!endDate.equals(signDate)) {
                            exceptionCount++;
                            dingFormResult.getSignInInfo().setStatus(DingStatusEnum.RE_SIGN);
                        }
                    }
                    if (dingFormResult.getSignOutInfo().getStatus().equals(DingStatusEnum.NO_SIGN)) {
                        if (!endDate.equals(signDate)) {
                            exceptionCount++;
                            dingFormResult.getSignOutInfo().setStatus(DingStatusEnum.RE_SIGN);
                        }
                    }
                    // 出勤天数增加
                    if ((dingFormResult.getSignInInfo().getStatus().equals(DingStatusEnum.SIGN) && dingFormResult.getSignOutInfo().getStatus().equals(DingStatusEnum.SIGN)) ||
                            (dingFormResult.getSignInInfo().getStatus().equals(DingStatusEnum.OTHER) && dingFormResult.getSignOutInfo().getStatus().equals(DingStatusEnum.OTHER))) {
                        workCount++;
                    }
                } else {
                    // 完全没有这天的打卡记录
                    DingInfoResult in = new DingInfoResult();
                    in.setStatus(DingStatusEnum.RE_SIGN);
                    DingInfoResult out = new DingInfoResult();
                    out.setStatus(DingStatusEnum.RE_SIGN);
                    dingFormResult.setSignInInfo(in);
                    dingFormResult.setSignOutInfo(out);
                    dingFormResult.setSignDate(signDate);
                    exceptionCount = exceptionCount + 2;
                }
                dingFormResult.setYear(year);
                formList.add(dingFormResult);
            }
        }
        SopCrmInfo sopCrmInfo = tbCrmService.findSopCrmInfoBySop(sopNo);
        Long serviceUnitId = sopCrmInfo.getServiceUnitId();
        DingElementResult dingElementResult = this.baseMapper.findDingResultByUserId(userId, serviceUnitId);
        int reissueCardCount = dingElementResult != null ? dingElementResult.getReissueCardCount() : 0;
        int applyCount = this.baseMapper.findDingExceptionApplyCount(userId, sopNo);

        DingStatisticResult dingStatisticResult = new DingStatisticResult();
        dingStatisticResult.setDingFormList(formList);
        dingStatisticResult.setWorkDays(workCount);
        dingStatisticResult.setExceptionCount(exceptionCount);
        dingStatisticResult.setRemainCount(reissueCardCount != 0 ? reissueCardCount - applyCount : 0);
        return dingStatisticResult;
    }

    @Override
    public void dingSave(DingSaveParam dingSaveParam, Long userId) throws IOException {
        String sopNo = dingSaveParam.getSopNo();
        SopCrmInfo sopCrmInfo = tbCrmService.findSopCrmInfoBySop(sopNo);
        Long serviceUnitId = sopCrmInfo.getServiceUnitId();
        DingElementResult dingElementResult = this.baseMapper.findDingResultByUserId(userId, serviceUnitId);

        Long signTime = dingSaveParam.getSignTime();
        DingDateTypeEnum dateType = sysDingDateService.getDingDateType(DateDisposeUtils.timestampToLocalDate(signTime));

        InOutTypeEnum signType = dingSaveParam.getSignType();
        String signAddress = dingSaveParam.getSignAddress();
        String facePhotoPath = dingSaveParam.getFacePhotoPath();
        String axisX = dingSaveParam.getAxisX();
        String axisY = dingSaveParam.getAxisY();

        TBUserArchives tbUserArchives = tbUserArchivesService.findByUserId(userId);
        Long userArchivesId = tbUserArchives.getId();
        // 人脸识别
        Boolean faceOpen = dingElementResult.getFaceOpen();
        BigDecimal score = null;
        Boolean facePass = null;
        if (faceOpen) {
            // 开启人脸识别
            if (facePhotoPath == null || facePhotoPath.length() == 0) {
                throw ExceptionResultEnum.ERROR.exception("缺少人脸识别照片");
            }
            String basePhotoPath = tbUserArchives.getBasePhotoPath();
            String basePhotoSuffix = basePhotoPath.substring(basePhotoPath.lastIndexOf(".")).toLowerCase();
            File basePhotoTemp = SystemConstant.getFileTempVar(basePhotoSuffix);
            String facePhotoSuffix = facePhotoPath.substring(facePhotoPath.lastIndexOf(".")).toLowerCase();
            File facePhotoTemp = SystemConstant.getFileTempVar(facePhotoSuffix);
            try {
                basePhotoTemp = fileStoreUtil.ossDownload(basePhotoPath, basePhotoTemp, UploadFileEnum.FILE.getFssType());
                facePhotoTemp = fileStoreUtil.ossDownload(facePhotoPath, facePhotoTemp, UploadFileEnum.FILE.getFssType());
                score = faceApiUtils.faceMatch(basePhotoTemp, facePhotoTemp);
            } catch (Exception e) {
                throw ExceptionResultEnum.ERROR.exception(e.getMessage());
            } finally {
                FileUtil.deleteFile(basePhotoTemp);
                FileUtil.deleteFile(facePhotoTemp);
            }
            String limitStr = sysConfigService.findByConfigKey(SystemConstant.BAIDU_FACE_COMPARE_SCORE).getConfigValue();
            BigDecimal limit = new BigDecimal(limitStr);
            facePass = score.compareTo(limit) > -1;
        }

        String crmNo = sopCrmInfo.getCrmNo();
        Long serviceId = sopCrmInfo.getServiceUnitId();
        Long flowId = sopCrmInfo.getFlowId();
        RoleTypeEnum roleType = sysRoleService.findRoleTypeByUserCrm(userId, crmNo);
        String signDate = DateFormatUtils.format(signTime, SystemConstant.DEFAULT_DATE_DAY_PATTERN);
        String currentDayStr = DateFormatUtils.format(System.currentTimeMillis(), SystemConstant.DEFAULT_DATE_DAY_PATTERN);
        if (!currentDayStr.equals(signDate)) {
            throw ExceptionResultEnum.ERROR.exception("打卡失败，时间异常");
        }

        // 新增打卡历史记录
        TBDingHistory history = new TBDingHistory();
        history.setSopNo(sopNo);
        history.setSignDate(signDate);
        history.setDateType(dateType);
        history.setSignTime(signTime);
        history.setAxisX(axisX);
        history.setAxisY(axisY);
        history.setFaceOpen(faceOpen);
        history.setFacePass(facePass);
        history.setFacePhotoPath(facePhotoPath);
        history.setMatchScore(score);
        history.setCreateId(userId);
        tbDingHistoryService.save(history);

        if (faceOpen && !facePass) {
            log.error("人脸识别不通过,score ：" + score);
            throw ExceptionResultEnum.ERROR.exception("人脸识别不通过!");
        }

        List<TBDing> tbDingList = this.list(new QueryWrapper<TBDing>().lambda()
                .eq(TBDing::getSopNo, sopNo)
                .eq(TBDing::getUserId, userId)
                .eq(TBDing::getSignDate, signDate));

        List<String> otherSopNoList = tbSopInfoService.findFlowPassageByCrmNo(crmNo);
        if (CollectionUtils.isEmpty(tbDingList)) {
            // 没有这天的打卡记录 - 新增打卡记录
            TBDing tbDing = new TBDing();
            tbDing.setSopNo(sopNo);
            tbDing.setDingSopNo(sopNo);
            tbDing.setCrmNo(crmNo);
            tbDing.setServiceId(serviceId);
            tbDing.setUserArchivesId(userArchivesId);
            tbDing.setRoleType(roleType);
            tbDing.setUserId(userId);
            tbDing.setFlowId(flowId);
            tbDing.setSignDate(signDate);
            tbDing.setDateType(dateType);
            switch (signType) {
                case IN: // 签到打卡
                    tbDing.setSignInTime(signTime);
                    tbDing.setSignInAddress(signAddress);
                    tbDing.setAxisInX(axisX);
                    tbDing.setAxisInY(axisY);
                    tbDing.setFaceInPass(facePass);
                    break;
                case OUT: // 签退打卡
                    tbDing.setSignOutTime(signTime);
                    tbDing.setSignOutAddress(signAddress);
                    tbDing.setAxisOutX(axisX);
                    tbDing.setAxisOutY(axisY);
                    tbDing.setFaceOutPass(facePass);
                    break;
            }
            List<TBDing> willAddDingList = otherSopNoList.stream().flatMap(e -> {
                TBDing cell = new TBDing();
                BeanUtils.copyProperties(tbDing, cell);
                cell.setSopNo(e);
                return Stream.of(cell);
            }).collect(Collectors.toList());
            this.saveBatch(willAddDingList);
        } else {
            // 有这天的打卡记录 - 签退打卡或更新打卡
            if (tbDingList.size() > 1) {
                throw ExceptionResultEnum.ERROR.exception(String.format("用户id为[%s],sop单号为[%s],在[%s]的打卡记录异常", userId, sopNo, signDate));
            }
            // 此人当天的所有sop打卡记录
            List<TBDing> recordList = this.list(new QueryWrapper<TBDing>().lambda()
                    .eq(TBDing::getDingSopNo, sopNo)
                    .eq(TBDing::getUserId, userId)
                    .eq(TBDing::getSignDate, signDate));

            switch (signType) {
                case IN: // 签到打卡
                    for (TBDing e : recordList) {
                        e.setSignInTime(signTime);
                        e.setSignInAddress(signAddress);
                        e.setAxisInX(axisX);
                        e.setAxisInY(axisY);
                        e.setFaceInPass(facePass);
                    }
                    break;
                case OUT: // 签退打卡
                    for (TBDing e : recordList) {
                        e.setSignOutTime(signTime);
                        e.setSignOutAddress(signAddress);
                        e.setAxisOutX(axisX);
                        e.setAxisOutY(axisY);
                        e.setFaceOutPass(facePass);
                    }
                    break;
            }
            this.updateBatchById(recordList);
        }
    }

    /**
     * 查询列表
     *
     * @param iPage
     * @return
     */

    @Override
    public IPage<TBDingSubmitQueryResult> query(IPage<Map> iPage, Long serviceId, String status, Long createId, String userName, Long supplierId, String custom, String sopNo, Long days) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        Long userId = requestUser.getId();
        DataPermissionDto dpr = sysUserService.buildUserDataPermission(userId);

        return this.baseMapper.query(iPage, serviceId, status, createId, userName, supplierId, custom, sopNo, days, dpr);
    }

    @Override
    public List<TBDingSubmitQueryResult> query(SysUser account, Long serviceId, String status, Long createId, String userName, Long supplierId, String custom, String sopNo, Long days) {
        Long userId = account.getId();
        DataPermissionDto dpr = sysUserService.buildUserDataPermission(userId);
        return this.baseMapper.query(serviceId, status, createId, userName, supplierId, custom, sopNo, days, dpr);
    }

    @Override
    public TBDingCountResult count(Long serviceId, Long createId, String userName, Long supplierId, String custom, String sopNo) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        Long userId = requestUser.getId();
        DataPermissionDto dpr = sysUserService.buildUserDataPermission(userId);
        return this.baseMapper.count(serviceId, createId, userName, supplierId, custom, sopNo, dpr);
    }

    @Override
    public IPage<TBDingAttendanceResult> attendanceQuery(IPage<Map> iPage, Long serviceId, String userName, Long startTime, Long endTime, Long supplierId, String custom, String sopNo, InOutTypeEnum type, DingResultEnum status) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        Long userId = requestUser.getId();
        DataPermissionDto dpr = sysUserService.buildUserDataPermission(userId);
        return this.baseMapper.attendanceQuery(iPage, serviceId, userName, startTime, endTime, supplierId, custom, sopNo, Objects.nonNull(type) ? type.name() : null, Objects.nonNull(status) ? status.name() : null, dpr);
    }

    @Override
    public TBDingAttendanceCountResult attendanceCount(Long serviceId, String userName, Long startTime, Long endTime, Long supplierId, String custom, String sopNo) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        Long userId = requestUser.getId();
        DataPermissionDto dpr = sysUserService.buildUserDataPermission(userId);
        return this.baseMapper.attendanceCount(serviceId, userName, startTime, endTime, supplierId, custom, sopNo, dpr);
    }

    @Override
    public List<TBDingAttendanceResult> attendanceQuery(Long serviceId, String userName, Long startTime, Long endTime, Long supplierId, String custom, String sopNo, InOutTypeEnum type, DingResultEnum status) {
        return this.baseMapper.attendanceQuery(serviceId, userName, startTime, endTime, supplierId, custom, sopNo, Objects.nonNull(type) ? type.name() : null, Objects.nonNull(status) ? status.name() : null);
    }

    @Override
    public DateFormDto buildSignDateFormBySop(String sopNo) {
        DateFormDto result = new DateFormDto();
        List<String> dateList = new ArrayList<>();
        if (sopNo != null && sopNo.length() > 0) {
            List<TFFlowApprove> tfFlowApproves = this.baseMapper.findDateAndStatusBySopNo(sopNo);
            if (CollectionUtils.isNotEmpty(tfFlowApproves)) {
                if (tfFlowApproves.size() > 1) {
                    throw ExceptionResultEnum.ERROR.exception(String.format("sop编号为[%s]的有多条流程审核记录", sopNo));
                }
                TFFlowApprove tfFlowApprove = tfFlowApproves.get(0);
                FlowStatusEnum flowStatusEnum = tfFlowApprove.getStatus();
                Long createTime = tfFlowApprove.getCreateTime();
                Long updateTime = tfFlowApprove.getUpdateTime();
                Long currentTime = System.currentTimeMillis();

                if (FlowStatusEnum.FINISH.equals(flowStatusEnum)) {
                    dateList = DateDisposeUtils.getDaysBetween(createTime, updateTime, SystemConstant.DEFAULT_DATE_YMD_S_PATTERN);
                    result.setSopIsEnd(true);
                } else {
                    dateList = DateDisposeUtils.getDaysBetween(createTime, currentTime, SystemConstant.DEFAULT_DATE_YMD_S_PATTERN);
                    result.setSopIsEnd(false);
                }
                dateList.sort(Collections.reverseOrder());
                result.setDateFormList(dateList);
            }
        }
        if (CollectionUtils.isNotEmpty(dateList)) {
            result.setEndDate(dateList.get(0));
        }
        return result;
    }
}